#!/usr/bin/env python
#####################################################################################
#
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://eu-egee.org/partners/ for details on the copyright holders.
# For license conditions see the license file or http://eu-egee.org/license.html
#
#####################################################################################
# glite-amga-server-config v. 1.0.1
#
# Post-installation script for configuring the gLite AMGA server
# Robert Harakaly  < robert.harakaly@cern.ch >
#
# Version info: $Id: glite-amga-server-config.py,v 1.12.2.3 2006/03/01 23:58:41 dimeglio Exp $
#
# Usage: python glite-file-transfer-agents-config [-c|-v|-h|--start|--stop|--status|--configure--help]
#        -c          print configuration
#        -v          print version
#        -h,--help   print usage info
#        --start     start the service
#        --stop      start the service
#        --status    get the service status (o=running, 1=idle) 
#        --configure configure the service and database (default)
#
# Return codes: 0 - Ok
#               1 - Configuration failed
#
#####################################################################################
from  gLiteInstallerLib import gLib
from  gLiteInstallerLib import ConfigParams
#from gliteRgmaServicetool import gliteRgmaServicetool
import os
import getopt
import sys

class glite_amga_server:
    def __init__(self, params):
        self.params = params
        self.verbose = 0
        self.version = "1.0.1"
        self.name = "glite-amga-server"
        self.friendly_name = "gLite AMGA server"
        params['module.version'] = self.version
        
    #-------------------------------------------------------------------------------
    # Banner 
    #-------------------------------------------------------------------------------

    def banner(self):

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "Configuring the %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
    
    #-------------------------------------------------------------------------------
    # Copyright 
    #-------------------------------------------------------------------------------

    def copyright(self):

        print '\nCopyright (c) Members of the EGEE Collaboration. 2004'
        print 'See http://eu-egee.org/partners/ for details on the copyright holders'
        print 'For license conditions see the license file or http://eu-egee.org/license.html'

    #-------------------------------------------------------------------------------
    # Version
    #-------------------------------------------------------------------------------

    def showVersion(self):

        print '\n%s-config  v. %s\n' % (self.name,self.version)
    
    #-------------------------------------------------------------------------------
    # Usage
    #-------------------------------------------------------------------------------

    def usage(self,msg = ""):

        if msg:
            print "\n%s" % (msg)
        
        self.copyright()
        self.showVersion()
    
        print """Usage: \n
Edit the configuration file %s.cfg.xml in
%s/etc.config/templates\n
save it as %s/etc/config/%s.cfg.xml
and run the script as follows\n 
python %s-config [OPTION...]""" % (self.name, os.environ['GLITE_LOCATION'], \
        os.environ['GLITE_LOCATION'], self.name, self.name)

        print '    -n, --noiptables    disables the automatic iptables configuration'
        print '    -c, --checkconf     print the service configuration'
        print '    -v, --version       print the version of the configuration script'
        print '    -h, --help          print this usage information'
        print '    --configure         configure the service'
        print '    --start             start the service'
        print '    --stop              stop the service'
        print '    --status            print service status (0 = running, 1 = idle)'
        print '    --siteconfig=<URL>  use site configuration file at URL'
        print '\n'


    def dbInit(self):
        if not params['amga.server.DataSource'] in ['sqlitedb','Oracle10g','PSQL','mysql']:
            glib.printErrorMessage('Database %s is not supported' % params['amga.server.DataSource'])
            return 1

        if params['amga.server.DataSource'] == 'sqlitedb':
            if not params.has_key('amga.server.DBDriverPath'):
                params['amga.server.DBDriverPath'] = '???' 
            if not params.has_key('amga.server.DBPort'):
                params['amga.server.DBPort'] = '???' 

            if not os.path.exists(params['amga.server.DBDriverPath']):
                glib.printErrorMessage('Database driver library %s is missing' % params['amga.server.DBDriverPath'])
                return 1
            
            os.system('sqlite3 dbfile.db < /opt/glite/share/doc/glite-amga-server-1.0.0/createInitial.sql')
            
        if params['amga.server.DataSource'] == 'Oracle10g':
            if not params.has_key('amga.server.DBDriverPath'):
                params['amga.server.DBDriverPath'] = '???' 
            if not params.has_key('amga.server.DBPort'):
                params['amga.server.DBPort'] = '???' 

            if not os.path.exists(params['amga.server.DBDriverPath']):
                glib.printErrorMessage('Database driver library %s is missing' % params['amga.server.DBDriverPath'])
                return 1

            os.system('sqlplus %s/%s@%s < /opt/glite/share/doc/glite-amga-server-1.0.0/createInitial.sql' % (params['amga.server.DBUser'], params['amga.server.DBPass'],\
                params['amga.server.DBUrl']))
                
        if params['amga.server.DataSource'] == 'PSQL':
            if not params.has_key('amga.server.DBDriverPath'):
                params['amga.server.DBDriverPath'] = '???' 
            if not params.has_key('amga.server.DBPort'):
                params['amga.server.DBPort'] = '???' 

            if not os.path.exists(params['amga.server.DBDriverPath']):
                glib.printErrorMessage('Database driver library %s is missing' % params['amga.server.DBDriverPath'])
                return 1


            os.system('psql -u %s db < /opt/glite/share/doc/glite-amga-server-1.0.0/createInitial.sql' % params['amga.server.DBUser'])


        if params['amga.server.DataSource'] == 'mysql':
            if not params.has_key('amga.server.DBDriverPath'):
                params['amga.server.DBDriverPath'] = '/usr/lib/libmyodbc3.so' 
            if not params.has_key('amga.server.DBPort'):
                params['amga.server.DBPort'] = '3306' 

            if not os.path.exists(params['amga.server.DBDriverPath']):
                glib.printErrorMessage('Database driver library %s is missing' % params['amga.server.DBDriverPath'])
                return 1
            mysql = os.popen('mysql -u %s -p%s -h %s %s < %s/share/doc/glite-amga-server-1.0.0/createInitialMySQL.sql'\
                % (params['amga.server.DBUser'], params['amga.server.DBPass'],\
               params['amga.server.DBHost'], params['amga.server.DBName'], params['GLITE_LOCATION'])) 
            ret = mysql.read()
            mysql.close()
            #print ret
            if ret.split(':')[0].startswith('ERROR') and not ret.split(':')[0] == 'ERROR 1050' :
                   glib.printErrorMessage('Schema upload has failed')
                   return 1
               
            # create odbc.ini
            ini = open("/etc/odbc.ini",'w')
            ini.write('[mysql]\n')
            
            ini.write('Trace    = Off\n')
            ini.write('TraceFile= stderr\n')
            ini.write('Driver   = %s\n' % params['amga.server.DBDriverPath'])
            ini.write('DSN      = %s\n' % params['amga.server.DBName'])
            ini.write('SERVER   = %s\n' % params['amga.server.DBHost'])
            ini.write('USER     = %s\n' % params['amga.server.DBUser'])
            ini.write('PASSWORD = %s\n' % params['amga.server.DBPass'])
            ini.write('PORT     = %s\n' % params['amga.server.DBPort'])
            ini.write('DATABASE = %s\n' % params['amga.server.DBName'])
            
            ini.close()
        return 0
        
    def configure(self):
        #-------------------------------------------------------------------------------
        # Installs the Security Utilities
        #--------------------------------------------------------
        
        if os.system("python %s/glite-security-utils-config.py --subservice" % glib.getScriptPath()):
            print "\nInstalling gLite Security Utilities               ",
            glib.printFailedMessage()
            return 1
        print "\nInstalling gLite Security Utilities                               ",
        glib.printOkMessage()
        if os.system("python %s/glite-amga-client-config.py --subservice" % glib.getScriptPath()):
            print "\nConfiguring amga client                          ",
            glib.printFailedMessage()
            return 1
        print "\nConfiguring amga client                               ",
        glib.printOkMessage()
        

        
        self.dbInit()
        glib.printInfoMessage("Creating mdserver.config file")
        conf = open ("%s/etc/mdserver.config" % params['GLITE_LOCATION'],"w")
        for param in self.params.keys():
            if param.startswith("amga.server"):
		if not params[param] == '':
                    conf.write("%s = %s\n" % (param.split(".")[2], params[param]))
        conf.close()
        return 0
        
    def start(self):
        return os.system("%s/etc/init.d/mdservice start" % params['GLITE_LOCATION'])
    def stop(self):
        return os.system("%s/etc/init.d/mdservice stop" % params['GLITE_LOCATION'])
    def status(self):
        return os.system("%s/etc/init.d/mdservice status" % params['GLITE_LOCATION'])

def setDefaults():
    params['amga.server.Port'] = 8822
    params['amga.server.MinProcesses'] = 2
    params['amga.server.MaxProcesses'] = 50
    params['amga.server.MaxConnectsPerProcess'] = ""
    params['amga.server.Sessions'] = "allow"
    params['amga.server.IdleTimeout'] = 1200
    params['amga.server.SessionTimeout'] = 86400
    params["amga.server.CertFile"] = params['host.certificate.file']
    params['amga.server.KeyFile'] = params['host.key.file']
    params['amga.server.TrustedCertDir'] = params['ca.certificates.dir']

    
    return 0
    
def set_env():
    glib.export('GLITE_LOCATION')
    glib.export('GLITE_LOCATION_VAR')
    glib.export('GLITE_LOCATION_LOG')
    glib.export('GLITE_LOCATION_TMP')
    glib.export('GLOBUS_LOCATION')
    glib.export('GPT_LOCATION')
    glib.export('LD_LIBRARY_PATH')
    glib.export('JAVA_HOME')
    
    glib.check_dir(os.environ['GLITE_LOCATION_VAR'])
    glib.check_dir(os.environ['GLITE_LOCATION_LOG'], 0777)
    glib.check_dir(os.environ['GLITE_LOCATION_TMP'])
    
    glib.addEnvPath("PATH", "%s/bin" % os.environ["JAVA_HOME"])
    glib.addEnvPath("PATH", "%s/externals/bin" % os.environ["GLITE_LOCATION"])
    glib.addEnvPath("PATH", "%s/bin" % os.environ["GLITE_LOCATION"])

    glib.addEnvPath("LD_LIBRARY_PATH", "/usr/local/lib")
    glib.addEnvPath("LD_LIBRARY_PATH", "%s/lib" % os.environ['GLOBUS_LOCATION'])
    glib.addEnvPath("LD_LIBRARY_PATH", "%s/externals/lib" % os.environ["GLITE_LOCATION"])
    glib.addEnvPath("LD_LIBRARY_PATH", "%s/lib" % os.environ["GLITE_LOCATION"])
    return 0
    
#-------------------------------------------------------------------------------
#  Main program begins here 
#-------------------------------------------------------------------------------

if __name__ == '__main__':

    # The script must be run as root
    if not os.geteuid() == 0:
        print '"\nThis script must be run as root\n'
        sys.exit(1)
        
    # Get an instance of the library class
    glib = gLib()
    glib.verbose = 0
    
    # Load parameters
    params = ConfigParams()
    try:
        opts, args = glib.getopt(sys.argv[1:], '', ['siteconfig='])
        for o, a in opts:
            if o == "--siteconfig":
                params['site.config.url'] = a
                break
    except getopt.GetoptError:
        pass
    if glib.loadConfiguration(["%s/../glite-amga-server.cfg.xml" % glib.getScriptPath(), \
            "%s/../glite-service-discovery.cfg.xml" % glib.getScriptPath()], params):
        print "An error occurred while configuring AMGA server"
        sys.exit(1)
    
    setDefaults()
    verbose = 0
    if params.has_key('glite.installer.verbose'):
        if params['glite.installer.verbose'] == "true":
            verbose = 1
    gLib.verbose = verbose
    
    # Set up the environment
    set_env()
          
    # Instantiate the service classes
    service = glite_amga_server(params)
    service.verbose = verbose

    # Instantiate the rgma servicetool class
#    if not params.has_key('rgma.servicetool.activate'):
#        params['rgma.servicetool.activate'] = 'true'
#    if params['rgma.servicetool.activate'] == 'true':
#        rgmaServicetool = gliteRgmaServicetool()
#        rgmaServicetool.verbose = verbose

    # Command line opts if any
    try:
        opts, args = getopt.getopt(sys.argv[1:], 'chv', ['checkconf', 'help', 'version', \
            'configure', 'remove', 'start', 'stop', 'status', 'siteconfig=', 'instance', 'restart'])
    except getopt.GetoptError:
        service.usage(msg = "Unknown options(s)")
        sys.exit(1)

    if len(opts) == 0:
        service.usage()
        sys.exit(0)
    
    # Check cli options
    for o, a in opts:

        if o in ("-h", "--help"):
            service.usage()
            sys.exit(0)

        if o in ("-v", "--version"):
            service.showVersion()
            sys.exit(0)

        if o in ("-c", "--checkconf"):
            service.copyright()
            service.showVersion()
            glib.print_params(params)
            sys.exit(0)

        if o == "--configure":
            # Check certificates
            if params.has_key('glite.installer.checkcerts'):
                if params['glite.installer.checkcerts'] == "true":
                    if glib.check_certs(params) != 0:
                        print "An error occurred while configuring the %s" \
                            % service.friendly_name
                        sys.exit(1)
            
            service.copyright()
            service.showVersion()
            service.banner()
                
            # Display the parameters if verbose output
            if verbose:
                glib.print_params(params)
                            
            # Configure the service
            return_value = service.configure()
            
            if return_value == 0:

                glib.registerService()

                print "\n\nThe %s configuration was successfully completed\n" % service.friendly_name
                print "You can now start the agents using the --start option of this script\n\n"
    
                sys.exit(0)

            elif return_value == 2:

                print "\n\nThe %s configuration was completed, but one or more warnings" % service.friendly_name
                print "were issued. Please revise them and re-run the script"
                print "or configure and start the %s manually\n" % service.friendly_name

                sys.exit(2)

            else:
                print "\n\nAn unrecoverable error occurred while configuring the %s" \
                    % service.friendly_name

                sys.exit(1)
            
        if o == "--restart":
            if service.stop() == 0:
                print "\n\nThe %s were successfully stopped\n" % service.friendly_name
                sys.exit(0)
            else:
                print "\n\nService %s did not stop correctly. For more\
                information consult the output of the script." % service.friendly_name
                sys.exit(1) 

            if service.start() == 0:
                print "\n\nThe %s were successfully started\n" % service.friendly_name
                sys.exit(0)
            else:
                print "\n\nService  %s did not start correctly. For more\
                information consult the output of the script." % service.friendly_name
                sys.exit(1)
            
        if o == "--start":
            if service.start() == 0:
                print "\n\nThe %s were successfully started\n" % service.friendly_name
                sys.exit(0)
            else:
                print "\n\nService %s did not start correctly. For more\
                information consult the output of the script." % service.friendly_name
                sys.exit(1)

        if o == "--stop":
            if service.stop() == 0:
                print "\n\nThe %s were successfully stopped\n" % service.friendly_name
                sys.exit(0)
            else:
                print "\n\nService %s did not stop correctly. For more\
                information consult the output of the script." % service.friendly_name
                sys.exit(1)

        if o == "--status":
            sys.exit(service.status())
